#!/usr/bin/env python
import logging, os, urllib2, urllib
from time import time
from modules.functions import VERSION
logging.basicConfig(level=logging.ERROR)
#from threading import Thread

from modules.clWidgets import MyLinkButton

try:
    from scrobbler import Scrobbler
except:
    logging.error('scrobbler not found')
    raise

try:
    import gtk
except:
    logging.error('gtk not found')
    raise

try:
    # for python 2.6
    from hashlib import md5
except ImportError:
    from md5 import md5

class main():
    ## The Constructor
    # @param self Object pointer
    # @param pyjama Reference to the pyjama object
    def __init__(self, pyjama):
    	self.pyjama = pyjama
        self.Events = self.pyjama.Events

        self.Events.connect_event("nowplaying", self.ev_nowplaying)
        self.Events.connect_event('player-status', self.ev_player_status)


        # login to libre.fm
        self.scrobbler = None

        if self.get_session():
            logging.debug('libre.fm plugin loaded')
        else:
            logging.debug('libre.fm plugin loaded but scrobbling isn\'t available')
        
            #raise

        self.last_scrobbled = None

#    def get_session(self):
#        thr = Thread(target = self.get_session_do, args = ())
#        thr.start()

    def get_session(self):
        if self.pyjama.settings.get_value('LIBREFM','SCROBBLING'):

            login=str(self.pyjama.settings.get_value('LIBREFM','LOGIN'))
            password=str(self.pyjama.settings.get_value('LIBREFM','PASS'))

            try:
                # pyjama has own libre.fm clien id 'pyj'
                self.scrobbler=Scrobbler(client=('pyj',VERSION))
                self.scrobbler.login(login, password)
            except Exception, e:
                logging.error(e)
                self.scrobbler=None
                return False
            else:
                #self.logged=True
                return True
        else:
            return False


#    def ev_nowplaying(self,track):
#        thr = Thread(target = self.ev_nowplaying_do, args = (track,))
#        thr.start()

    def ev_nowplaying(self, track):
        """ for sending now playing notification to libre.fm """
        logging.debug('nowplaying: %s' % track)

        if self.scrobbler is None:
            self.get_session()

        if self.pyjama.settings.get_value('LIBREFM','SCROBBLING') and self.scrobbler is not None:
            try:
                self.scrobbler.now_playing(track.artist_name, track.name, album=track.album_name, length=track.duration)
            except (self.scrobbler.SessionError, self.scrobbler.AuthError), e:
                logging.error('can\'t send nowplaying info to libre.fm')
                self.get_session()
            except Exception, e:
                logging.error(e)
            else:
                logging.info('nowplaying %s - %s send to libre.fm' % (track.artist_name, track.name))
                self.pyjama.setInfo("librefm - nowplaying send")

        else:
            logging.debug('nowplaying info doesn\'t send - scrobbling is off')

#    def ev_scrobble(self,track):
#        thr = Thread(target = self.ev_scrobble_do, args = (track,))
#        thr.start()

    def ev_scrobble(self,track):
        """ for scrobbling to libre.fm """
        logging.debug('track: %s' % track)

        if self.scrobbler is None:
            self.get_session()

        if self.pyjama.settings.get_value('LIBREFM','SCROBBLING') and self.scrobbler is not None:

            try:
                self.scrobbler.submit(track.artist_name,track.name,int(time()),length=track.duration)
                self.scrobbler.flush()
            except (self.scrobbler.SessionError, self.scrobbler.AuthError), e:
                logging.warn('libre.fm plugin: can\'t scrobble song')
                self.get_session()
            except Exception, e:
                logging.error(e)
                #raise
            else:
                logging.info('song %s - %s send to libre.fm' % (track.artist_name, track.name))
                self.pyjama.setInfo("librefm - scrobbled")

        else:
            logging.debug('song doesn\' scrobbled - scrobbling is off')

    def ev_player_status(self, status, cursec = None, duration = None):
        if status == "playing":
            percentage =  (1.0 / duration) * cursec
            if percentage > 1: percentage = 0

            # scrobble test
            if (percentage > 0.5 or (cursec > 240 and cursec <= duration)) and duration > 30 and self.pyjama.player.cur_playing.id != self.last_scrobbled:
                #print "send this to libre.fm ;) <<<< ", self.cursec, '/',  self.duration
                self.ev_scrobble(self.pyjama.player.cur_playing)
                self.last_scrobbled = self.pyjama.player.cur_playing.id

    class Preferences():
        ## Preferences Constructor
        # @param self Object Pointer
        # @param pyjama Again we have a pyjama object reference passed to our class
        def __init__(self, pyjama):
            self.pyjama = pyjama
            # get settings
            login_value=str(pyjama.settings.get_value('LIBREFM','LOGIN','login'))
            pass_value=str(pyjama.settings.get_value('LIBREFM','PASS', 'password'))
            scrobble_value=pyjama.settings.get_value('LIBREFM','SCROBBLING',False)

            logging.debug('settings on start: %s:%s %s' % (login_value,pass_value,str(scrobble_value)))
            
            dialog = gtk.Dialog(buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
            dialog.set_title(_("libre.fm Plugin Preferences"))
            dialog.set_default_size(100,100)

            hbox = gtk.HBox()
            dialog.vbox.pack_start(hbox, False, True)
            hbox.show()

            hbox2 = gtk.HBox()
            dialog.vbox.pack_start(hbox2, False, True)
            hbox2.show()


            # scrobble check button
            check = gtk.CheckButton(_("Scrobble to libre.fm?"))
            check.set_active(scrobble_value)
            dialog.vbox.pack_start(check, False, True)

            # login field
            llabel=gtk.Label(_('Login:'))

            llabel.set_property('has-tooltip', True)
            llabel.set_property('tooltip-text', 'aaaaaa')
            hbox.pack_start(llabel, False, True)

            if login_value != "login":
                lbl = MyLinkButton("http://alpha.libre.fm/user/%s"  % login_value, "Open my page @librefm")
                lbl.show()
                lbl.connect("clicked", self.cb_lbl_clicked)
                dialog.vbox.pack_end(lbl, False, True)

            login = gtk.Entry(max=0)
            login.set_text(login_value)
            hbox.pack_end(login, False, True)

            # password field
            plabel=gtk.Label(_('Password:'))
            hbox2.pack_start(plabel, False, True)

            password = gtk.Entry(max=0)
            password.set_visibility(False)            
            password.set_text(pass_value)

            password.set_visibility(False)            

            hbox2.pack_end(password, False, True)

            dialog.show_all()
            result = dialog.run()

            # set new settings
            if result == gtk.RESPONSE_ACCEPT:
                pyjama.settings.set_value('LIBREFM','LOGIN', str(login.get_text()))
                logging.debug('new login: %s' % login.get_text())

                pyjama.settings.set_value('LIBREFM','PASS',str(password.get_text()))
                logging.debug('new pass: %s' % password.get_text())

                pyjama.settings.set_value('LIBREFM','SCROBBLING',str(check.get_active()))
                logging.debug('new scrobble vaule: %s' % check.get_active())

            dialog.destroy()

        def cb_lbl_clicked(self, widget):
            self.pyjama.Events.raise_event("open_url", widget.uri, True)



